@@ -2,32 +2,36 @@
 # Copyright (c) 2012 Ry4an Brase <ry4an-hg@ry4an.org>
 # Copyright (c) 2012 Google, Inc.
 # Copyright (c) 2012 Anthony VEREZ <anthony.verez.external@cassidian.com>
-# Copyright (c) 2014-2018 Claudiu Popa <pcmanticore@gmail.com>
+# Copyright (c) 2014-2019 Claudiu Popa <pcmanticore@gmail.com>
 # Copyright (c) 2014 Brett Cannon <brett@python.org>
 # Copyright (c) 2014 Arun Persaud <arun@nubati.net>
 # Copyright (c) 2015 Ionel Cristian Maries <contact@ionelmc.ro>
-# Copyright (c) 2017 Anthony Sottile <asottile@umich.edu>
+# Copyright (c) 2017, 2020 Anthony Sottile <asottile@umich.edu>
 # Copyright (c) 2017 Mikhail Fesenko <proggga@gmail.com>
+# Copyright (c) 2018 Scott Worley <scottworley@scottworley.com>
 # Copyright (c) 2018 ssolanki <sushobhitsolanki@gmail.com>
+# Copyright (c) 2019 Hugo van Kemenade <hugovk@users.noreply.github.com>
+# Copyright (c) 2019 Taewon D. Kim <kimt33@mcmaster.ca>
+# Copyright (c) 2019 Pierre Sassoulas <pierre.sassoulas@gmail.com>
 
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 # For details: https://github.com/PyCQA/pylint/blob/master/COPYING
 
-# pylint: disable=W0622
+# pylint: disable=redefined-builtin
 """a similarities / code duplication command line tool and pylint checker
 """
 
-from __future__ import print_function
 import sys
 from collections import defaultdict
+from getopt import getopt
 from itertools import groupby
 
 import astroid
 
-from pylint.utils import decoding_stream
-from pylint.interfaces import IRawChecker
 from pylint.checkers import BaseChecker, table_lines_from_stats
+from pylint.interfaces import IRawChecker
 from pylint.reporters.ureports.nodes import Table
+from pylint.utils import decoding_stream
 
 
 class Similar:
@@ -96,11 +100,12 @@ class Similar:
             print()
             print(num, "similar lines in", len(couples), "files")
             couples = sorted(couples)
+            lineset = idx = None
             for lineset, idx in couples:
                 print("==%s:%s" % (lineset.name, idx))
-            # pylint: disable=W0631
-            for line in lineset._real_lines[idx : idx + num]:
-                print("  ", line.rstrip())
+            if lineset:
+                for line in lineset._real_lines[idx : idx + num]:
+                    print("  ", line.rstrip())
             nb_lignes_dupliquees += num * (len(couples) - 1)
         nb_total_lignes = sum([len(lineset) for lineset in self.linesets])
         print(
@@ -108,7 +113,7 @@ class Similar:
             % (
                 nb_total_lignes,
                 nb_lignes_dupliquees,
-                nb_lignes_dupliquees * 100. / nb_total_lignes,
+                nb_lignes_dupliquees * 100.0 / nb_total_lignes,
             )
         )
 
@@ -148,8 +153,7 @@ class Similar:
         """
         for idx, lineset in enumerate(self.linesets[:-1]):
             for lineset2 in self.linesets[idx + 1 :]:
-                for sim in self._find_common(lineset, lineset2):
-                    yield sim
+                yield from self._find_common(lineset, lineset2)
 
 
 def stripped_lines(lines, ignore_comments, ignore_docstrings, ignore_imports):
@@ -175,7 +179,9 @@ def stripped_lines(lines, ignore_comments, ignore_docstrings, ignore_imports):
     for lineno, line in enumerate(lines, start=1):
         line = line.strip()
         if ignore_docstrings:
-            if not docstring and (line.startswith('"""') or line.startswith("'''")):
+            if not docstring and any(
+                line.startswith(i) for i in ['"""', "'''", 'r"""', "r'''"]
+            ):
                 docstring = line[:3]
                 line = line[3:]
             if docstring:
@@ -189,8 +195,6 @@ def stripped_lines(lines, ignore_comments, ignore_docstrings, ignore_imports):
             if current_line_is_import:
                 line = ""
         if ignore_comments:
-            # XXX should use regex in checkers/format to avoid cutting
-            # at a "#" in a string
             line = line.split("#", 1)[0].strip()
         strippedlines.append(line)
     return strippedlines
@@ -377,16 +381,19 @@ class SimilarChecker(BaseChecker, Similar):
         stats = self.stats
         for num, couples in self._compute_sims():
             msg = []
+            lineset = idx = None
             for lineset, idx in couples:
                 msg.append("==%s:%s" % (lineset.name, idx))
             msg.sort()
-            # pylint: disable=W0631
-            for line in lineset._real_lines[idx : idx + num]:
-                msg.append(line.rstrip())
+
+            if lineset:
+                for line in lineset._real_lines[idx : idx + num]:
+                    msg.append(line.rstrip())
+
             self.add_message("R0801", args=(len(couples), "\n".join(msg)))
             duplicated += num * (len(couples) - 1)
         stats["nb_duplicated_lines"] = duplicated
-        stats["percent_duplicated_lines"] = total and duplicated * 100. / total
+        stats["percent_duplicated_lines"] = total and duplicated * 100.0 / total
 
 
 def register(linter):
@@ -409,7 +416,6 @@ def Run(argv=None):
     """standalone command line access point"""
     if argv is None:
         argv = sys.argv[1:]
-    from getopt import getopt
 
     s_opts = "hdi"
     l_opts = (
