@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # Copyright (c) 2013-2014 LOGILAB S.A. (Paris, FRANCE) <contact@logilab.fr>
 # Copyright (c) 2013-2014 Google, Inc.
-# Copyright (c) 2014-2018 Claudiu Popa <pcmanticore@gmail.com>
+# Copyright (c) 2014-2019 Claudiu Popa <pcmanticore@gmail.com>
 # Copyright (c) 2014 Cosmin Poieana <cmin@ropython.org>
 # Copyright (c) 2014 Vlad Temian <vladtemian@gmail.com>
 # Copyright (c) 2014 Arun Persaud <arun@nubati.net>
@@ -13,8 +13,16 @@
 # Copyright (c) 2017 Martin <MartinBasti@users.noreply.github.com>
 # Copyright (c) 2017 Christopher Zurcher <zurcher@users.noreply.github.com>
 # Copyright (c) 2017 Łukasz Rogalski <rogalski.91@gmail.com>
+# Copyright (c) 2018 Lucas Cimon <lucas.cimon@gmail.com>
 # Copyright (c) 2018 Banjamin Freeman <befreeman@users.noreply.github.com>
 # Copyright (c) 2018 Ioana Tagirta <ioana.tagirta@gmail.com>
+# Copyright (c) 2019 Julien Palard <julien@palard.fr>
+# Copyright (c) 2019 laike9m <laike9m@users.noreply.github.com>
+# Copyright (c) 2019 Hugo van Kemenade <hugovk@users.noreply.github.com>
+# Copyright (c) 2019 Robert Schweizer <robert_schweizer@gmx.de>
+# Copyright (c) 2019 fadedDexofan <fadedDexofan@gmail.com>
+# Copyright (c) 2019 Pierre Sassoulas <pierre.sassoulas@gmail.com>
+# Copyright (c) 2020 Anthony Sottile <asottile@umich.edu>
 
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 # For details: https://github.com/PyCQA/pylint/blob/master/COPYING
@@ -149,7 +157,7 @@ class StdlibChecker(BaseChecker):
             "subprocess-run-check",
             "The check parameter should always be used with explicitly set "
             "`check` keyword to make clear what the error-handling behavior is."
-            "https://docs.python.org/3/library/subprocess.html#subprocess.runs",
+            "https://docs.python.org/3/library/subprocess.html#subprocess.run",
         ),
     }
 
@@ -233,9 +241,7 @@ class StdlibChecker(BaseChecker):
             (3, 4, 4): {"asyncio.tasks.async"},
             (3, 5, 0): {
                 "fractions.gcd",
-                "inspect.getargvalues",
                 "inspect.formatargspec",
-                "inspect.formatargvalues",
                 "inspect.getcallargs",
                 "platform.linux_distribution",
                 "platform.dist",
