@@ -3,7 +3,7 @@
 # Copyright (c) 2010-2014 LOGILAB S.A. (Paris, FRANCE) <contact@logilab.fr>
 # Copyright (c) 2010 Daniel Harding <dharding@gmail.com>
 # Copyright (c) 2012-2014 Google, Inc.
-# Copyright (c) 2013-2018 Claudiu Popa <pcmanticore@gmail.com>
+# Copyright (c) 2013-2020 Claudiu Popa <pcmanticore@gmail.com>
 # Copyright (c) 2014 Brett Cannon <brett@python.org>
 # Copyright (c) 2014 Arun Persaud <arun@nubati.net>
 # Copyright (c) 2015 Rene Zhang <rz99@cornell.edu>
@@ -12,8 +12,17 @@
 # Copyright (c) 2016 Peter Dawyndt <Peter.Dawyndt@UGent.be>
 # Copyright (c) 2017 Łukasz Rogalski <rogalski.91@gmail.com>
 # Copyright (c) 2017 Ville Skyttä <ville.skytta@iki.fi>
+# Copyright (c) 2018, 2020 Anthony Sottile <asottile@umich.edu>
+# Copyright (c) 2018-2019 Lucas Cimon <lucas.cimon@gmail.com>
+# Copyright (c) 2018 Alan Chan <achan961117@gmail.com>
+# Copyright (c) 2018 Yury Gribov <tetra2005@gmail.com>
+# Copyright (c) 2018 ssolanki <sushobhitsolanki@gmail.com>
 # Copyright (c) 2018 Nick Drozd <nicholasdrozd@gmail.com>
-# Copyright (c) 2018 Anthony Sottile <asottile@umich.edu>
+# Copyright (c) 2019 Wes Turner <westurner@google.com>
+# Copyright (c) 2019 Djailla <bastien.vallet@gmail.com>
+# Copyright (c) 2019 Hugo van Kemenade <hugovk@users.noreply.github.com>
+# Copyright (c) 2019 Pierre Sassoulas <pierre.sassoulas@gmail.com>
+# Copyright (c) 2020 Anthony <tanant@users.noreply.github.com>
 
 
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
@@ -23,9 +32,12 @@
 """
 
 import builtins
+import collections
 import numbers
+import re
 import tokenize
-from collections import Counter
+import typing
+from typing import Iterable
 
 import astroid
 from astroid.arguments import CallSite
@@ -36,6 +48,37 @@ from pylint.checkers.utils import check_messages
 from pylint.interfaces import IAstroidChecker, IRawChecker, ITokenChecker
 
 _AST_NODE_STR_TYPES = ("__builtin__.unicode", "__builtin__.str", "builtins.str")
+# Prefixes for both strings and bytes literals per
+# https://docs.python.org/3/reference/lexical_analysis.html#string-and-bytes-literals
+_PREFIXES = {
+    "r",
+    "u",
+    "R",
+    "U",
+    "f",
+    "F",
+    "fr",
+    "Fr",
+    "fR",
+    "FR",
+    "rf",
+    "rF",
+    "Rf",
+    "RF",
+    "b",
+    "B",
+    "br",
+    "Br",
+    "bR",
+    "BR",
+    "rb",
+    "rB",
+    "Rb",
+    "RB",
+}
+SINGLE_QUOTED_REGEX = re.compile("(%s)?'''" % "|".join(_PREFIXES))
+DOUBLE_QUOTED_REGEX = re.compile('(%s)?"""' % "|".join(_PREFIXES))
+QUOTE_DELIMITER_REGEX = re.compile("(%s)?(\"|')" % "|".join(_PREFIXES), re.DOTALL)
 
 MSGS = {
     "E1300": (
@@ -152,6 +195,12 @@ MSGS = {
         "Used when we detect that a string formatting is "
         "repeating an argument instead of using named string arguments",
     ),
+    "W1309": (
+        "Using an f-string that does not have any interpolated variables",
+        "f-string-without-interpolation",
+        "Used when we detect an f-string that does not use any interpolation variables, "
+        "in which case it can be either a normal string or a bug in the code.",
+    ),
 }
 
 OTHER_NODES = (
@@ -209,7 +258,19 @@ class StringFormatChecker(BaseChecker):
     msgs = MSGS
 
     # pylint: disable=too-many-branches
-    @check_messages(*MSGS)
+    @check_messages(
+        "bad-format-character",
+        "truncated-format-string",
+        "mixed-format-string",
+        "bad-format-string-key",
+        "missing-format-string-key",
+        "unused-format-string-key",
+        "bad-string-format-type",
+        "format-needs-mapping",
+        "too-many-format-args",
+        "too-few-format-args",
+        "bad-string-format-type",
+    )
     def visit_binop(self, node):
         if node.op != "%":
             return
@@ -220,9 +281,12 @@ class StringFormatChecker(BaseChecker):
             return
         format_string = left.value
         try:
-            required_keys, required_num_args, required_key_types, required_arg_types = utils.parse_format_string(
-                format_string
-            )
+            (
+                required_keys,
+                required_num_args,
+                required_key_types,
+                required_arg_types,
+            ) = utils.parse_format_string(format_string)
         except utils.UnsupportedFormatCharacter as exc:
             formatted = format_string[exc.index]
             self.add_message(
@@ -333,6 +397,15 @@ class StringFormatChecker(BaseChecker):
                             args=(arg_type.pytype(), format_type),
                         )
 
+    @check_messages("f-string-without-interpolation")
+    def visit_joinedstr(self, node):
+        if isinstance(node.parent, astroid.FormattedValue):
+            return
+        for value in node.values:
+            if isinstance(value, astroid.FormattedValue):
+                return
+        self.add_message("f-string-without-interpolation", node=node)
+
     @check_messages(*MSGS)
     def visit_call(self, node):
         func = utils.safe_infer(node.func)
@@ -355,7 +428,7 @@ class StringFormatChecker(BaseChecker):
                 self._check_new_format(node, func)
 
     def _detect_vacuous_formatting(self, node, positional_arguments):
-        counter = Counter(
+        counter = collections.Counter(
             arg.name for arg in positional_arguments if isinstance(arg, astroid.Name)
         )
         for name, count in counter.items():
@@ -562,12 +635,19 @@ class StringConstantChecker(BaseTokenChecker):
             "Used when an escape like \\u is encountered in a byte "
             "string where it has no effect.",
         ),
-        "W1403": (
+        "W1404": (
             "Implicit string concatenation found in %s",
-            "implicit-str-concat-in-sequence",
+            "implicit-str-concat",
             "String literals are implicitly concatenated in a "
             "literal iterable definition : "
             "maybe a comma is missing ?",
+            {"old_names": [("W1403", "implicit-str-concat-in-sequence")]},
+        ),
+        "W1405": (
+            "Quote delimiter %s is inconsistent with the rest of the file",
+            "inconsistent-quotes",
+            "Quote delimiters are not used consistently throughout a module "
+            "(with allowances made for avoiding unnecessary escaping).",
         ),
     }
     options = (
@@ -578,11 +658,22 @@ class StringConstantChecker(BaseTokenChecker):
                 "type": "yn",
                 "metavar": "<y_or_n>",
                 "help": "This flag controls whether the "
-                "implicit-str-concat-in-sequence should generate a warning "
+                "implicit-str-concat should generate a warning "
                 "on implicit string concatenation in sequences defined over "
                 "several lines.",
             },
         ),
+        (
+            "check-quote-consistency",
+            {
+                "default": False,
+                "type": "yn",
+                "metavar": "<y_or_n>",
+                "help": "This flag controls whether inconsistent-quotes generates a "
+                "warning when the character used as a quote delimiter is used "
+                "inconsistently within a module.",
+            },
+        ),
     )
 
     # Characters that have a special meaning after a backslash in either
@@ -594,7 +685,7 @@ class StringConstantChecker(BaseTokenChecker):
     UNICODE_ESCAPE_CHARACTERS = "uUN"
 
     def __init__(self, *args, **kwargs):
-        super(StringConstantChecker, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         self.string_tokens = {}  # token position -> (token value, next token)
 
     def process_module(self, module):
@@ -625,43 +716,83 @@ class StringConstantChecker(BaseTokenChecker):
                     start = (start[0], len(line[: start[1]].encode(encoding)))
                 self.string_tokens[start] = (str_eval(token), next_token)
 
-    @check_messages(*(msgs.keys()))
+        if self.config.check_quote_consistency:
+            self.check_for_consistent_string_delimiters(tokens)
+
+    @check_messages("implicit-str-concat")
     def visit_list(self, node):
-        self.check_for_concatenated_strings(node, "list")
+        self.check_for_concatenated_strings(node.elts, "list")
 
-    @check_messages(*(msgs.keys()))
+    @check_messages("implicit-str-concat")
     def visit_set(self, node):
-        self.check_for_concatenated_strings(node, "set")
+        self.check_for_concatenated_strings(node.elts, "set")
 
-    @check_messages(*(msgs.keys()))
+    @check_messages("implicit-str-concat")
     def visit_tuple(self, node):
-        self.check_for_concatenated_strings(node, "tuple")
+        self.check_for_concatenated_strings(node.elts, "tuple")
 
-    def check_for_concatenated_strings(self, iterable_node, iterable_type):
-        for elt in iterable_node.elts:
-            if isinstance(elt, Const) and elt.pytype() in _AST_NODE_STR_TYPES:
-                if elt.col_offset < 0:
-                    # This can happen in case of escaped newlines
+    def visit_assign(self, node):
+        if isinstance(node.value, astroid.Const) and isinstance(node.value.value, str):
+            self.check_for_concatenated_strings([node.value], "assignment")
+
+    def check_for_consistent_string_delimiters(
+        self, tokens: Iterable[tokenize.TokenInfo]
+    ) -> None:
+        """Adds a message for each string using inconsistent quote delimiters.
+
+        Quote delimiters are used inconsistently if " and ' are mixed in a module's
+        shortstrings without having done so to avoid escaping an internal quote
+        character.
+
+        Args:
+          tokens: The tokens to be checked against for consistent usage.
+        """
+        string_delimiters = collections.Counter()  # type: typing.Counter[str]
+
+        # First, figure out which quote character predominates in the module
+        for tok_type, token, _, _, _ in tokens:
+            if tok_type == tokenize.STRING and _is_quote_delimiter_chosen_freely(token):
+                string_delimiters[_get_quote_delimiter(token)] += 1
+
+        if len(string_delimiters) > 1:
+            # Ties are broken arbitrarily
+            most_common_delimiter = string_delimiters.most_common(1)[0][0]
+            for tok_type, token, start, _, _ in tokens:
+                if tok_type != tokenize.STRING:
                     continue
-                if (elt.lineno, elt.col_offset) not in self.string_tokens:
-                    # This may happen with Latin1 encoding
-                    # cf. https://github.com/PyCQA/pylint/issues/2610
-                    continue
-                matching_token, next_token = self.string_tokens[
-                    (elt.lineno, elt.col_offset)
-                ]
-                # We detect string concatenation: the AST Const is the
-                # combination of 2 string tokens
-                if matching_token != elt.value and next_token is not None:
-                    if next_token.type == tokenize.STRING and (
-                        next_token.start[0] == elt.lineno
-                        or self.config.check_str_concat_over_line_jumps
-                    ):
-                        self.add_message(
-                            "implicit-str-concat-in-sequence",
-                            line=elt.lineno,
-                            args=(iterable_type,),
-                        )
+                quote_delimiter = _get_quote_delimiter(token)
+                if (
+                    _is_quote_delimiter_chosen_freely(token)
+                    and quote_delimiter != most_common_delimiter
+                ):
+                    self.add_message(
+                        "inconsistent-quotes", line=start[0], args=(quote_delimiter,)
+                    )
+
+    def check_for_concatenated_strings(self, elements, iterable_type):
+        for elt in elements:
+            if not (isinstance(elt, Const) and elt.pytype() in _AST_NODE_STR_TYPES):
+                continue
+            if elt.col_offset < 0:
+                # This can happen in case of escaped newlines
+                continue
+            if (elt.lineno, elt.col_offset) not in self.string_tokens:
+                # This may happen with Latin1 encoding
+                # cf. https://github.com/PyCQA/pylint/issues/2610
+                continue
+            matching_token, next_token = self.string_tokens[
+                (elt.lineno, elt.col_offset)
+            ]
+            # We detect string concatenation: the AST Const is the
+            # combination of 2 string tokens
+            if matching_token != elt.value and next_token is not None:
+                if next_token.type == tokenize.STRING and (
+                    next_token.start[0] == elt.lineno
+                    or self.config.check_str_concat_over_line_jumps
+                ):
+                    self.add_message(
+                        "implicit-str-concat", line=elt.lineno, args=(iterable_type,)
+                    )
 
     def process_string_token(self, token, start_row):
         quote_char = None
@@ -753,3 +884,69 @@ def str_eval(token):
     if token[0:3] in ('"""', "'''"):
         return token[3:-3]
     return token[1:-1]
+
+
+def _is_long_string(string_token: str) -> bool:
+    """Is this string token a "longstring" (is it triple-quoted)?
+
+    Long strings are triple-quoted as defined in
+    https://docs.python.org/3/reference/lexical_analysis.html#string-and-bytes-literals
+
+    This function only checks characters up through the open quotes.  Because it's meant
+    to be applied only to tokens that represent string literals, it doesn't bother to
+    check for close-quotes (demonstrating that the literal is a well-formed string).
+
+    Args:
+        string_token: The string token to be parsed.
+
+    Returns:
+        A boolean representing whether or not this token matches a longstring
+        regex.
+    """
+    return bool(
+        SINGLE_QUOTED_REGEX.match(string_token)
+        or DOUBLE_QUOTED_REGEX.match(string_token)
+    )
+
+
+def _get_quote_delimiter(string_token: str) -> str:
+    """Returns the quote character used to delimit this token string.
+
+    This function does little checking for whether the token is a well-formed
+    string.
+
+    Args:
+        string_token: The token to be parsed.
+
+    Returns:
+        A string containing solely the first quote delimiter character in the passed
+        string.
+
+    Raises:
+      ValueError: No quote delimiter characters are present.
+    """
+    match = QUOTE_DELIMITER_REGEX.match(string_token)
+    if not match:
+        raise ValueError("string token %s is not a well-formed string" % string_token)
+    return match.group(2)
+
+
+def _is_quote_delimiter_chosen_freely(string_token: str) -> bool:
+    """Was there a non-awkward option for the quote delimiter?
+
+    Args:
+        string_token: The quoted string whose delimiters are to be checked.
+
+    Returns:
+        Whether there was a choice in this token's quote character that would
+        not have involved backslash-escaping an interior quote character.  Long
+        strings are excepted from this analysis under the assumption that their
+        quote characters are set by policy.
+    """
+    quote_delimiter = _get_quote_delimiter(string_token)
+    unchosen_delimiter = '"' if quote_delimiter == "'" else "'"
+    return bool(
+        quote_delimiter
+        and not _is_long_string(string_token)
+        and unchosen_delimiter not in str_eval(string_token)
+    )
