@@ -1,20 +1,18 @@
-# Copyright (c) 2016 Claudiu Popa <pcmanticore@gmail.com>
+# Copyright (c) 2016, 2018 Claudiu Popa <pcmanticore@gmail.com>
+# Copyright (c) 2019 Hugo van Kemenade <hugovk@users.noreply.github.com>
+# Copyright (c) 2019 Pierre Sassoulas <pierre.sassoulas@gmail.com>
 
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 # For details: https://github.com/PyCQA/pylint/blob/master/COPYING
 
 """Checker for deprecated builtins."""
-import sys
-
 import astroid
+
 from pylint.checkers import BaseChecker
 from pylint.checkers.utils import check_messages
 from pylint.interfaces import IAstroidChecker
 
-
 BAD_FUNCTIONS = ["map", "filter"]
-if sys.version_info < (3, 0):
-    BAD_FUNCTIONS.append("input")
 # Some hints regarding the use of bad builtins.
 BUILTIN_HINTS = {"map": "Using a list comprehension can be clearer."}
 BUILTIN_HINTS["filter"] = BUILTIN_HINTS["map"]
