@@ -1,29 +1,30 @@
 # -*- coding: utf-8 -*-
 # Copyright (c) 2014-2015 Bruno Daniel <bruno.daniel@blue-yonder.com>
-# Copyright (c) 2015-2017 Claudiu Popa <pcmanticore@gmail.com>
-# Copyright (c) 2016-2018 Ashley Whetter <ashley@awhetter.co.uk>
+# Copyright (c) 2015-2019 Claudiu Popa <pcmanticore@gmail.com>
+# Copyright (c) 2016-2019 Ashley Whetter <ashley@awhetter.co.uk>
 # Copyright (c) 2016 Glenn Matthews <glenn@e-dad.net>
 # Copyright (c) 2016 Glenn Matthews <glmatthe@cisco.com>
 # Copyright (c) 2016 Moises Lopez <moylop260@vauxoo.com>
 # Copyright (c) 2017 Ville Skyttä <ville.skytta@iki.fi>
 # Copyright (c) 2017 John Paraskevopoulos <io.paraskev@gmail.com>
-# Copyright (c) 2018 Anthony Sottile <asottile@umich.edu>
+# Copyright (c) 2018, 2020 Anthony Sottile <asottile@umich.edu>
+# Copyright (c) 2018 Jim Robertson <jrobertson98atx@gmail.com>
 # Copyright (c) 2018 Sushobhit <31987769+sushobhit27@users.noreply.github.com>
 # Copyright (c) 2018 Adam Dangoor <adamdangoor@gmail.com>
+# Copyright (c) 2019 Hugo van Kemenade <hugovk@users.noreply.github.com>
+# Copyright (c) 2019 Pierre Sassoulas <pierre.sassoulas@gmail.com>
 
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 # For details: https://github.com/PyCQA/pylint/blob/master/COPYING
 
 """Pylint plugin for checking in Sphinx, Google, or Numpy style docstrings
 """
-from __future__ import print_function, division, absolute_import
-
 import astroid
 
-from pylint.interfaces import IAstroidChecker
+import pylint.extensions._check_docs_utils as utils
 from pylint.checkers import BaseChecker
 from pylint.checkers import utils as checker_utils
-import pylint.extensions._check_docs_utils as utils
+from pylint.interfaces import IAstroidChecker
 
 
 class DocstringParameterChecker(BaseChecker):
@@ -78,7 +79,7 @@ class DocstringParameterChecker(BaseChecker):
             "Missing return documentation",
             "missing-return-doc",
             "Please add documentation about what this method returns.",
-            {"old_names": [("W9007", "missing-returns-doc")]},
+            {"old_names": [("W9007", "old-missing-returns-doc")]},
         ),
         "W9012": (
             "Missing return type documentation",
@@ -91,7 +92,7 @@ class DocstringParameterChecker(BaseChecker):
             "Missing yield documentation",
             "missing-yield-doc",
             "Please add documentation about what this generator yields.",
-            {"old_names": [("W9009", "missing-yields-doc")]},
+            {"old_names": [("W9009", "old-missing-yields-doc")]},
         ),
         "W9014": (
             "Missing yield type documentation",
@@ -104,13 +105,13 @@ class DocstringParameterChecker(BaseChecker):
             '"%s" missing in parameter documentation',
             "missing-param-doc",
             "Please add parameter declarations for all parameters.",
-            {"old_names": [("W9003", "missing-param-doc")]},
+            {"old_names": [("W9003", "old-missing-param-doc")]},
         ),
         "W9016": (
             '"%s" missing in parameter type documentation',
             "missing-type-doc",
             "Please add parameter type declarations for all parameters.",
-            {"old_names": [("W9004", "missing-type-doc")]},
+            {"old_names": [("W9004", "old-missing-type-doc")]},
         ),
         "W9017": (
             '"%s" differing in parameter documentation',
@@ -252,6 +253,7 @@ class DocstringParameterChecker(BaseChecker):
             return
 
         expected_excs = utils.possible_exc_types(node)
+
         if not expected_excs:
             return
 
@@ -268,8 +270,11 @@ class DocstringParameterChecker(BaseChecker):
                 self._handle_no_raise_doc(expected_excs, func_node)
             return
 
-        found_excs = doc.exceptions()
-        missing_excs = expected_excs - found_excs
+        found_excs_full_names = doc.exceptions()
+
+        # Extract just the class name, e.g. "error" from "re.error"
+        found_excs_class_names = {exc.split(".")[-1] for exc in found_excs_full_names}
+        missing_excs = expected_excs - found_excs_class_names
         self._add_raise_message(missing_excs, func_node)
 
     def visit_return(self, node):
@@ -314,12 +319,79 @@ class DocstringParameterChecker(BaseChecker):
         if not doc_has_yields:
             self.add_message("missing-yield-doc", node=func_node)
 
-        if not doc_has_yields_type:
+        if not (doc_has_yields_type or func_node.returns):
             self.add_message("missing-yield-type-doc", node=func_node)
 
     def visit_yieldfrom(self, node):
         self.visit_yield(node)
 
+    def _compare_missing_args(
+        self,
+        found_argument_names,
+        message_id,
+        not_needed_names,
+        expected_argument_names,
+        warning_node,
+    ):
+        """Compare the found argument names with the expected ones and
+        generate a message if there are arguments missing.
+
+        :param set found_argument_names: argument names found in the
+            docstring
+
+        :param str message_id: pylint message id
+
+        :param not_needed_names: names that may be omitted
+        :type not_needed_names: set of str
+
+        :param set expected_argument_names: Expected argument names
+        :param NodeNG warning_node: The node to be analyzed
+        """
+        missing_argument_names = (
+            expected_argument_names - found_argument_names
+        ) - not_needed_names
+        if missing_argument_names:
+            self.add_message(
+                message_id,
+                args=(", ".join(sorted(missing_argument_names)),),
+                node=warning_node,
+            )
+
+    def _compare_different_args(
+        self,
+        found_argument_names,
+        message_id,
+        not_needed_names,
+        expected_argument_names,
+        warning_node,
+    ):
+        """Compare the found argument names with the expected ones and
+        generate a message if there are extra arguments found.
+
+        :param set found_argument_names: argument names found in the
+            docstring
+
+        :param str message_id: pylint message id
+
+        :param not_needed_names: names that may be omitted
+        :type not_needed_names: set of str
+
+        :param set expected_argument_names: Expected argument names
+        :param NodeNG warning_node: The node to be analyzed
+        """
+        differing_argument_names = (
+            (expected_argument_names ^ found_argument_names)
+            - not_needed_names
+            - expected_argument_names
+        )
+
+        if differing_argument_names:
+            self.add_message(
+                message_id,
+                args=(", ".join(sorted(differing_argument_names)),),
+                node=warning_node,
+            )
+
     def check_arguments_in_docstring(
         self, doc, arguments_node, warning_node, accept_no_param_doc=None
     ):
@@ -342,7 +414,7 @@ class DocstringParameterChecker(BaseChecker):
           and the absence is tolerated.
 
         :param doc: Docstring for the function, method or class.
-        :type doc: str
+        :type doc: :class:`Docstring`
 
         :param arguments_node: Arguments node for the function, method or
             class constructor.
@@ -382,71 +454,44 @@ class DocstringParameterChecker(BaseChecker):
         if not params_with_doc and not params_with_type and accept_no_param_doc:
             tolerate_missing_params = True
 
-        def _compare_missing_args(found_argument_names, message_id, not_needed_names):
-            """Compare the found argument names with the expected ones and
-            generate a message if there are arguments missing.
-
-            :param set found_argument_names: argument names found in the
-                docstring
-
-            :param str message_id: pylint message id
-
-            :param not_needed_names: names that may be omitted
-            :type not_needed_names: set of str
-            """
-            if not tolerate_missing_params:
-                missing_argument_names = (
-                    expected_argument_names - found_argument_names
-                ) - not_needed_names
-                if missing_argument_names:
-                    self.add_message(
-                        message_id,
-                        args=(", ".join(sorted(missing_argument_names)),),
-                        node=warning_node,
-                    )
-
-        def _compare_different_args(found_argument_names, message_id, not_needed_names):
-            """Compare the found argument names with the expected ones and
-            generate a message if there are extra arguments found.
-
-            :param set found_argument_names: argument names found in the
-                docstring
-
-            :param str message_id: pylint message id
-
-            :param not_needed_names: names that may be omitted
-            :type not_needed_names: set of str
-            """
-            differing_argument_names = (
-                (expected_argument_names ^ found_argument_names)
-                - not_needed_names
-                - expected_argument_names
+        if not tolerate_missing_params:
+            self._compare_missing_args(
+                params_with_doc,
+                "missing-param-doc",
+                self.not_needed_param_in_docstring,
+                expected_argument_names,
+                warning_node,
             )
 
-            if differing_argument_names:
-                self.add_message(
-                    message_id,
-                    args=(", ".join(sorted(differing_argument_names)),),
-                    node=warning_node,
-                )
-
-        _compare_missing_args(
-            params_with_doc, "missing-param-doc", self.not_needed_param_in_docstring
-        )
-
         for index, arg_name in enumerate(arguments_node.args):
             if arguments_node.annotations[index]:
                 params_with_type.add(arg_name.name)
+        for index, arg_name in enumerate(arguments_node.kwonlyargs):
+            if arguments_node.kwonlyargs_annotations[index]:
+                params_with_type.add(arg_name.name)
 
-        _compare_missing_args(
-            params_with_type, "missing-type-doc", not_needed_type_in_docstring
-        )
+        if not tolerate_missing_params:
+            self._compare_missing_args(
+                params_with_type,
+                "missing-type-doc",
+                not_needed_type_in_docstring,
+                expected_argument_names,
+                warning_node,
+            )
 
-        _compare_different_args(
-            params_with_doc, "differing-param-doc", self.not_needed_param_in_docstring
+        self._compare_different_args(
+            params_with_doc,
+            "differing-param-doc",
+            self.not_needed_param_in_docstring,
+            expected_argument_names,
+            warning_node,
         )
-        _compare_different_args(
-            params_with_type, "differing-type-doc", not_needed_type_in_docstring
+        self._compare_different_args(
+            params_with_type,
+            "differing-type-doc",
+            not_needed_type_in_docstring,
+            expected_argument_names,
+            warning_node,
         )
 
     def check_single_constructor_params(self, class_doc, init_doc, class_node):
